import click
import os
import shutil
import subprocess
from pathlib import Path
from pyfiglet import figlet_format
from colorama import init
from termcolor import cprint


MODULES_FOLDER = ["vpc", "cluster", "instance"]
MODULES_FILES = ["main.tf", "variables.tf", "outputs.tf"]
PROJECT_FILES = [
    "main.tf",
    "variables.tf",
    "outputs.tf",
    "backend.tf",
    "main.tfvars",
    ".gitignore",
]


def print_version(ctx, param, value):
    """Print current version"""
    if not value or ctx.resilient_parsing:
        return
    click.echo("Version: 1.4")
    ctx.exit()


def create_modules():
    """Create the modules directory and its subdirectories and files"""
    Path("modules").mkdir()
    for folder in MODULES_FOLDER:
        Path(f"modules/{folder}").mkdir()
        os.chdir(f"modules/{folder}")
        for file in MODULES_FILES:
            with open(file, "w+") as handler:
                pass
        os.chdir("../..")


def create_env_folders(env):
    """Create the specified environment folders"""
    for e in env:
        Path(e).mkdir()
        click.echo(f"Created {e} environment.")


def create_env_files(env):
    """Create the terraform files for each environment"""
    for e in env:
        os.chdir(e)
        for file in PROJECT_FILES:
            with open(file, "w+") as handler:
                pass
        os.chdir("..")


@click.group()
@click.option(
    "--version",
    "-v",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help="Show version",
)
def main():
    """ClI tool to create Terraform project"""
    pass


@main.command()
@click.option(
    "--name", "-n", help="Name of the project", prompt=True, confirmation_prompt=True
)
@click.option(
    "--env",
    "-e",
    help="Environment name(dev, stg, prd)",
    prompt=True,
    multiple=True,
)
def create(name, env):
    """Create a new Terraform project with specified name and environment"""
    try:
        Path(name).mkdir()
        os.chdir(name)
        create_modules()
        create_env_folders(env)
        create_env_files(env)
        click.secho(
            (f"Created project {name} successfully, You're ready move to ☁️ !!"),
            fg="green",
        )
    except FileExistsError:
        click.secho(("The directory already exists!"), bg="red")
        os.chdir("..")


@main.command()
@click.option(
    "--name", "-n", help="Name of the project", prompt=True, confirmation_prompt=True
)
def remove(name):
    """Delete the project"""
    project_dir = Path(name)
    if project_dir.is_dir():
        click.confirm(
            click.style(f"Are you sure you want to delete {name}?", fg="blue"),
            abort=True,
        )
        shutil.rmtree(project_dir)
        click.secho(f"Project {name} deleted successfully", fg="green")
    else:
        click.secho(f"Project {name} not found", fg="red")


@main.command()
@click.option(
    "--name", "-n", help="Name of the project", prompt=True, confirmation_prompt=True
)
def check(name):
    """Check all files inside a Terraform project are valid."""
    project_dir = Path(name)
    if not project_dir.is_dir():
        click.secho(f"Project {name} not found", fg="red")
        return
    os.chdir(name)
    success = True
    results = []
    for root, dirs, files in os.walk("."):
        for file in files:
            if file.endswith(".tf"):
                try:
                    subprocess.check_output(
                        ["terraform", "validate", os.path.join(root, file)]
                    )
                    results.append(f"✔️ {os.path.join(root, file)}")
                except subprocess.CalledProcessError as e:
                    success = False
                    results.append(
                        f"❌ {os.path.join(root, file)} - {e.output.decode('utf-8').strip()}"
                    )
    if success:
        print("Check succeeded")
    else:
        print("Check failed with the following results:")
        for result in results:
            print(result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['terramagic']
install_requires = \
['click>=8.0.3,<9.0.0',
 'colorama>=0.4.4,<0.5.0',
 'pyfiglet>=0.8.post1,<0.9',
 'python-hcl2>=4.3.0,<5.0.0',
 'termcolor>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['terramagic = terramagic:main']}

setup_kwargs = {
    'name': 'terramagic',
    'version': '0.1.4',
    'description': 'A automate tool for terraform projects',
    'long_description': '# Terramagic CLI\n\n\n## Motivation\n\nEvery time , I needed create a terraform files to a new project, and a new terraform files., but this is not good. and now we have a Terramagic tool to help us to create a terraform files.\n\n## Requirements\n\n- Python 3.9 >=\n\n## How to install?\n\n```shell\npip install terramagic\n```\n\n## Hands on\n\n## Check the version\n\n\n## Usage(Ex.)\n\n```shell\nterramagic create-project --name <project_name> --env <env>\n```\n\n```shell\nterramagic create-project --name terraform --env prod --env dev\n```\n\n## How to use this tool ?\n\n```shell\nUsage: terramagic create-project [OPTIONS]\n\n\nOptions:\n  -n, --name TEXT  Name of the project\n  -e, --env TEXT   Environment name(dev, test, prd)\n  --help           Show this message and exit.\n```\n\nEnjoy!\n',
    'author': 'Milton Jesus',
    'author_email': 'milton.lima@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/miltlima/terramagic',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
